from pydantic import BaseSettings
from pydantic.fields import Field
from telethon import TelegramClient
from typing import List

__all__ = ["config"]


class Config(BaseSettings):
    telegram_app_id: str = Field(None, env="TELEGRAM_APP_ID", required=True)

    telegram_api_hash: str = Field(None, env="TELEGRAM_API_HASH", required=True)

    telegram_account_phone: str = Field(
        None, env="TELEGRAM_ACCOUNT_PHONE", required=True
    )

    app_mode: str = Field(name="app_mode", default="dev", env="APP_MODE")

    debug_mode: bool = False

    flask_port: str = Field(name="flask_port", default="8080", env="FLASK_PORT")

    telegram_client: TelegramClient = None
    
    excluded_chats: List[str] = Field([], env="BOT_EXCLUDED_CHATS")

    class Config:
        env_file = '.env'
        env_file_encoding = 'utf-8'

config = Config()
