import asyncio
import logging
from multiprocessing import Manager, Process

from clockblocker_bot.config import config
from clockblocker_bot.server_app import create_web_app
from telethon import TelegramClient

logger = logging.getLogger(__name__)


def setup_logging():
    logging.basicConfig(
        format="[%(levelname)8s] %(msg)s {%(filename)s:%(lineno)s}", level=logging.DEBUG
    )
    logging.getLogger("telethon").setLevel(logging.WARNING)


def get_code_from_web(token):
    async def waiter():
        while token.value is None:
            await asyncio.sleep(1)
        logger.info("Token received!")
        return token.value

    return waiter


def init():
    setup_logging()
    logger.debug("Initializing client")
    print('config', config)
    client = TelegramClient(
        "bot_session", config.telegram_app_id, config.telegram_api_hash
    )
    manager = Manager()
    token = manager.Value("s", None)
    web_app = Process(target=create_web_app, args=(token,))
    web_app.start()
    client.start(
        phone=config.telegram_account_phone, code_callback=get_code_from_web(token)
    )
    config.telegram_client = client
    from clockblocker_bot.actions import finish

    finish()

    return client


def main():
    client = init()
    client.run_until_disconnected()
