import asyncio
import functools
import logging
from typing import Union

from clockblocker_bot.utils.constants import CLOCKBLOCKER_ID, S3RIUS_ID
from telethon import events, functions, types

logger = logging.getLogger(__name__)

chat_union = Union[types.User, types.Chat, types.Channel]

def ignore_with_id(*ids):
    def ignore_inner(f):
        @functools.wraps(f)
        async def wrapper(event: events.NewMessage.Event):

            try:
                if not event.message.from_id.user_id in ids:
                    await f(event)
            except Exception as e:
                logger.error(f"Exception found: {e}")

        return wrapper
    return ignore_inner

def only_with_id(id):
    def only_inner(f):
        @functools.wraps(f)
        async def wrapper(event: events.NewMessage.Event):
            try:
                if event.message.from_id.user_id == id:
                    await f(event)

            except Exception as e:
                logger.error(f"Exception found: {e}")

        return wrapper
    return only_inner


def only_clockblocker(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        try:
            # print('event.message.from_id == CLOCKBLOCKER_ID', event.message.from_id == CLOCKBLOCKER_ID, event.message.from_id, CLOCKBLOCKER_ID)
            if event.message.from_id.user_id == CLOCKBLOCKER_ID:
                await f(event)

        except Exception as e:
            logger.error(f"Exception found: {e}")

    return wrapper


def mark_unread(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        try:
            await f(event)
        except Exception as e:
            logger.error(f"Exception found: {e}")
        chat = await event.get_chat()
        functions.messages.MarkDialogUnreadRequest(chat, unread=True)

    return wrapper


def clean_up_policy(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        try:
            reply_msg: types.Message = await f(event)
            print('\n', reply_msg, '\n')
            await asyncio.sleep(10)
            await reply_msg.delete()
        except Exception as e:
            logger.error(f"Exception found: {e}")

    return wrapper


def beware_the_croud_policy(telegram_client):
    print(telegram_client)
    def beware_the_croud_policy_inner(f):
        @functools.wraps(f)
        async def wrapper(event: events.NewMessage.Event):
            try:
                ok = True
                chat: chat_union = await event.get_chat()
                participants = []
                async for participant in telegram_client.iter_participants(chat):
                    participants.append(participant)
                if isinstance(chat, types.Chat) or isinstance(chat, types.Channel):
                    ok = len(participants) < 50
                if ok:
                    await f(event)

            except Exception as e:
                logger.error(f"Exception found: {e}")

        return wrapper
    return beware_the_croud_policy_inner

def only_alkochat(f):
    @functools.wraps(f)
    async def wrapper(event: events.NewMessage.Event):
        try:
            alkochat_id = 1391173925
            chat: chat_union = await event.get_chat()
            if isinstance(chat, types.Chat) and chat.id == alkochat_id:
                await f(event)

        except Exception as e:
            logger.error(f"Exception found: {e}")

    return wrapper
    
